@@ -1,4 +1,4 @@
-# (c) 2003-2013 Vlado Keselj http://web.cs.dal.ca/~vlado
+# (c) 2003-2017 Vlado Keselj http://web.cs.dal.ca/~vlado
 #
 # Text::Ngrams - A Perl module for N-grams processing
 
@@ -12,11 +12,10 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 %EXPORT_TAGS = ( 'all' => [ qw(new encode_S decode_S) ] );
 @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 @EXPORT = qw(new);
-$VERSION = '2.005';
+$VERSION = '2.006';
 
-use vars qw($Version $Revision);
+use vars qw($Version);
 $Version = $VERSION;
-($Revision = substr(q$Revision: 399 $, 10)) =~ s/\s+$//;
 
 use vars @EXPORT_OK;
 use vars qw();			# non-exported package globals go here
@@ -50,25 +49,28 @@ sub new {
   #MJ ->
   #this type is analogous to the "character" type but defined for utf8 characters
   elsif ($params{type} eq 'utf8_character') {
-      #$self->{inputlayer}        
-      #input layer to be put on the input stream by the function binmode before reading from a given stream
-      #and to be removed by ***binmode HANDLE,":pop"*** after the reading from the particular stream is done
-      #has to be a real layer (like ":encoding(utf8)"), not a pseudo layer (like ":utf8")
-      #so that the psuedo layer ":pop" is able to remove this input layer 
-      
-      $self->{inputlayer}=':encoding(utf8)';
-      #this will automatically decode input text from utf8 into Perl internal reporesentation of Unicode strings
-      #and so the regular expressions for Unicode as well as the uc function can be performed on them
+    # $self->{inputlayer}
+    # input layer to be put on the input stream by the function binmode
+    # before reading from a given stream and to be removed by
+    # ***binmode HANDLE,":pop"*** after the reading from the particular
+    # stream is done has to be a real layer (like ":encoding(utf8)"), not a
+    # pseudo layer (like ":utf8") so that the psuedo layer ":pop" is able to
+    # remove this input layer 
       
+    $self->{inputlayer} = ':encoding(utf8)';
+    # this will automatically decode input text from utf8 into Perl internal
+    # reporesentation of Unicode strings and so the regular expressions for
+    # Unicode as well as the uc function can be performed on them
+
+    $self->{skiprex} = '';
      
-      $self->{skiprex} = '';
-     
-      $self->{tokenrex} = qr/(\p{IsAlpha}|\P{IsAlpha}+)/;
+    $self->{tokenrex} = qr/(\p{IsAlpha}|\P{IsAlpha}+)/;
                  
-      $self->{processtoken} =  sub { s/\P{IsAlpha}+/ /; $_ = uc $_ ; $_=Encode::encode_utf8( $_ );};
-      #the last operation ***$_=Encode::encode_utf8( $_ )*** is necessary 
-      #to go back to utf8 encoding from the internal Perl representation
-      #so that for the output the n-grams are in utf8 (encoded by encode_S though)
+    $self->{processtoken} =  sub { s/\P{IsAlpha}+/ /; $_ = uc $_ ;
+				   $_ = Encode::encode_utf8( $_ ); };
+    # the last operation ***$_=Encode::encode_utf8( $_ )*** is necessary 
+    # to go back to utf8 encoding from the internal Perl representation
+    # so that for the output the n-grams are in utf8 (encoded by encode_S though)
       
       $self->{allow_iproc} = 0;
       # allow_iproc has to be 0. Otherwise the last token in the read block will
@@ -242,6 +244,7 @@ sub process_files {
 	if (not ref($f))
 	{ open(F, "$f") or die "cannot open $f:$!"; $f1 = *F }
 	else { $f1 = $f }
+	binmode $f1;  # avoid text mode
 
 	#MJ ->
 	#put the encoding layer on the input when requested
@@ -510,10 +513,12 @@ Text::Ngrams - Flexible Ngram analysis (for characters, words, and more)
 For default character n-gram analysis of string:
 
   use Text::Ngrams;
-  my $ng3 = Text::Ngrams->new;
-  $ng3->process_text('abcdefg1235678hijklmnop');
+  my $ng = Text::Ngrams->new;
+  $ng->process_text('abcdefg1235678hijklmnop');
   print $ng3->to_string;
-  my @ngramsarray = $ng3->get_ngrams;
+  my @ngramsarray = $ng->get_ngrams;
+  # or put ngrams and frequencies into a hash
+  my %ngrams = $ng3->get_ngrams( n => 3, normalize => 1 );
 
 One can also feed tokens manually:
 
@@ -521,12 +526,8 @@ One can also feed tokens manually:
   my $ng3 = Text::Ngrams->new;
   $ng3->feed_tokens('a');
   $ng3->feed_tokens('b');
-  $ng3->feed_tokens('c');
-  $ng3->feed_tokens('d');
-  $ng3->feed_tokens('e');
-  $ng3->feed_tokens('f');
-  $ng3->feed_tokens('g');
-  $ng3->feed_tokens('h');
+  $ng3->feed_tokens('c', 'd');
+  $ng3->feed_tokens(qw(e f g h));
 
 We can choose n-grams of various sizes, e.g.:
 
@@ -804,8 +805,9 @@ foolowing way:
 =head2 feed_tokens ( list of tokens )
 
   $ng3->feed_tokens('a');
+  $ng3->feed_tokens('b', 'c');
 
-This function manually supplies tokens.
+This function supplies tokens directly.
 
 =head2 process_text ( list of strings )
 
@@ -816,11 +818,16 @@ Process text, i.e., break each string into tokens and feed them.
 
 =head2 process_files ( file_names or file_handle_references)
 
+A usage example:
+
   $ng->process_files('somefile.txt');
 
-Process files, similarly to text.
-The files are processed line by line, so there should not be any
-multi-line tokens.
+This method is used to process one or more files, similarly to processing text.
+The files are processed line by line, so there should be no multi-line tokens.
+Instead of filenames we can also give as arguments file handle references
+when a file is already open.  In this way, we can use the standard input handle as in:
+
+  $ng->process_files(\*STDIN);
 
 =head2 get_ngrams ( n => NUMBER, orderby => 'ngram|frequency|none', onlyfirst => NUMBER, out => filename|handle,normalize=>1)
 
@@ -979,7 +986,7 @@ that you used the module.
 
 Author:
 
- 2003-2013 Vlado Keselj http://web.cs.dal.ca/~vlado
+ 2003-2017 Vlado Keselj http://web.cs.dal.ca/~vlado
 
 Contributors:
 
@@ -990,6 +997,15 @@ This module is provided "as is" without expressed or implied warranty.
 This is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
+To acknowledge the use of this module in academic publications, please use
+a reference to the following paper:
+
+N-gram-based Author Profiles for Authorship Attribution.
+Vlado Keselj, Fuchun Peng, Nick Cercone, and Calvin Thomas. In Proceedings of
+the Conference Pacific Association for Computational Linguistics, PACLING'03,
+Dalhousie University, Halifax, Nova Scotia, Canada, pp. 255-264, August 2003. 
+http://web.cs.dal.ca/~vlado/papers/meta/Kes03.html
+
 The latest version can be found at F<http://web.cs.dal.ca/~vlado/srcperl/>.
 
 =head1 HISTORY
@@ -1037,4 +1053,4 @@ The page is available
 Some links to these resources should be available at F<http://web.cs.dal.ca/~vlado/nlp>.
 
 =cut
-# $Id: Ngrams.pm 399 2013-02-17 16:01:35Z vlado $
+# $Id: $
